@@ -4,7 +4,12 @@
 """Implements jsonlib.write"""
 
 from decimal import Decimal
-from .util import memoized
+import array
+import collections
+import UserList
+import UserDict
+import UserString
+from .util import memoized, INFINITY
 from . import errors
 
 __all__ = ['write']
@@ -26,6 +31,9 @@ ESCAPES = {
 	'\\': '\\\\'
 }
 
+for char_ord in range (0, 0x20):
+	ESCAPES.setdefault (chr (char_ord), '\\u%04x' % char_ord)
+	
 def get_indent (indent_string, indent_level):
 	if indent_string is None:
 		return '', '', ''
@@ -35,7 +43,8 @@ def write_array (value, sort_keys, indent_string, ascii_only, coerce_keys,
                  parent_objects, indent_level):
 	"""Serialize an iterable to a list of strings in JSON array format."""
 	
-	if value in parent_objects:
+	v_id = id (value)
+	if v_id in parent_objects:
 		raise errors.WriteError ("Can't write self-referential values.")
 		
 	newline, indent, next_indent = get_indent (indent_string, indent_level)
@@ -46,22 +55,30 @@ def write_array (value, sort_keys, indent_string, ascii_only, coerce_keys,
 			retval.append (indent)
 		retval.extend (_write (item, sort_keys, indent_string,
 		                       ascii_only, coerce_keys,
-		                       parent_objects + (value,),
+		                       parent_objects + (v_id,),
 		                       indent_level + 1))
 		if (index + 1) < len (value):
 			if newline:
 				retval.append (',' + newline)
 			else:
-				retval.append (', ')
+				retval.append (',')
 	retval.append (newline + next_indent)
 	retval.append (']')
 	return retval
 	
+def write_generator (value, *args, **kwargs):
+	return write_array (tuple (value), *args, **kwargs)
+	
+def write_unordered_array (value, sort_keys, *args, **kwargs):
+	return write_array (sorted (value) if sort_keys else value,
+	                    sort_keys, *args, **kwargs)
+	
 def write_object (value, sort_keys, indent_string, ascii_only, coerce_keys,
                   parent_objects, indent_level):
 	"""Serialize a mapping to a list of strings in JSON object format."""
 	
-	if value in parent_objects:
+	v_id = id (value)
+	if v_id in parent_objects:
 		raise errors.WriteError ("Can't write self-referential values.")
 		
 	newline, indent, next_indent = get_indent (indent_string, indent_level)
@@ -75,7 +92,7 @@ def write_object (value, sort_keys, indent_string, ascii_only, coerce_keys,
 	if newline:
 		separator = ',' + newline
 	else:
-		separator = ', '
+		separator = ','
 	for index, (key, sub_value) in enumerate (items):
 		is_string = isinstance (key, str)
 		is_unicode = isinstance (key, unicode)
@@ -95,10 +112,13 @@ def write_object (value, sort_keys, indent_string, ascii_only, coerce_keys,
 			raise errors.WriteError ("Only strings may "
 			                         "be used as object "
 			                         "keys.")
-		retval.append (': ')
+		if newline:
+			retval.append (': ')
+		else:
+			retval.append (':')
 		retval.extend (_write (sub_value, sort_keys, indent_string,
 		                       ascii_only, coerce_keys,
-		                       parent_objects + (value,),
+		                       parent_objects + (v_id,),
 		                       indent_level + 1))
 		if (index + 1) < len (value):
 			retval.append (separator)
@@ -112,10 +132,6 @@ def write_char (char, ascii_only):
 	if char in ESCAPES:
 		return ESCAPES[char]
 		
-	# Control character
-	if ord (char) in range (0x0, 0x1F + 1):
-		return '\\u%04x' % ord (char)
-		
 	# Unicode
 	if ord (char) > 0x7E and ascii_only:
 		# Split into surrogate pairs
@@ -152,31 +168,56 @@ def write_unicode (value, ascii_only):
 	
 @memoized
 def write_float (value):
-	disallowed = ('inf', '-inf', 'nan', 'Infinity', '-Infinity', 'NaN')
+	if value != value:
+		raise errors.WriteError ("Cannot serialize NaN.")
+	if value == INFINITY:
+		raise errors.WriteError ("Cannot serialize Infinity.")
+	if value == -INFINITY:
+		raise errors.WriteError ("Cannot serialize -Infinity.")
+	return repr (value)
+	
+def write_decimal (value):
+	if value != value:
+		raise errors.WriteError ("Cannot serialize NaN.")
 	s_value = unicode (value)
-	if s_value in disallowed:
-		raise errors.WriteError ("Cannot write floating-point value %r" % value)
+	if s_value in ('Infinity', '-Infinity'):
+		raise errors.WriteError ("Cannot serialize %r." % value)
 	return s_value
 	
+def write_complex (value):
+	if value.imag == 0.0:
+		return unicode (value.real)
+	raise errors.WriteError ("Cannot serialize complex numbers with"
+	                         " imaginary components.")
+	
 # Fundamental types
 _m_str = memoized (unicode)
 CONTAINER_TYPES = {
 	dict: write_object,
 	list: write_array,
 	tuple: write_array,
+	UserList.UserList: write_array,
+	UserDict.UserDict: write_object,
+	collections.deque: write_array,
+	array.array: write_array,
+	set: write_unordered_array,
+	frozenset: write_unordered_array,
+	type ((_ for _ in ())): write_generator,
 }
 
 STR_TYPE_MAPPERS = {
 	unicode: write_unicode,
 	str: write_string,
+	UserString.UserString: write_string,
 }
 
 TYPE_MAPPERS = {
 	int: _m_str,
 	long: _m_str,
 	float: write_float,
-	Decimal: write_float,
-	type (True): (lambda val: 'true' if val else 'false'),
+	complex: write_complex,
+	Decimal: write_decimal,
+	bool: (lambda val: 'true' if val else 'false'),
 	type (None): lambda _: 'null',
 }
 
@@ -184,19 +225,19 @@ def write_basic (value, ascii_only):
 	v_type = type (value)
 	if v_type in STR_TYPE_MAPPERS:
 		return STR_TYPE_MAPPERS[v_type] (value, ascii_only)
-	elif v_type in TYPE_MAPPERS:
+	if v_type in TYPE_MAPPERS:
 		return TYPE_MAPPERS[v_type] (value)
-	else:
-		# Might be a subclass
-		for mapper_type, mapper in STR_TYPE_MAPPERS.items ():
-			if isinstance (value, mapper_type):
-				return mapper (value, ascii_only)
-		for mapper_type, mapper in TYPE_MAPPERS.items ():
-			if isinstance (value, mapper_type):
-				return mapper (value)
-				
-		raise errors.UnknownSerializerError (value)
 		
+	# Might be a subclass
+	for mapper_type, mapper in STR_TYPE_MAPPERS.items ():
+		if isinstance (value, mapper_type):
+			return mapper (value, ascii_only)
+	for mapper_type, mapper in TYPE_MAPPERS.items ():
+		if isinstance (value, mapper_type):
+			return mapper (value)
+			
+	raise errors.UnknownSerializerError (value)
+	
 def _write (value, sort_keys, indent_string, ascii_only, coerce_keys,
             parent_objects, indent_level):
 	"""Serialize a Python value into a list of byte strings.
@@ -204,12 +245,21 @@ def _write (value, sort_keys, indent_string, ascii_only, coerce_keys,
 	When joined together, result in the value's JSON representation.
 	
 	"""
-	v_type = type (value)
-	if v_type in CONTAINER_TYPES:
-		w_func = CONTAINER_TYPES[v_type]
+	w_func = CONTAINER_TYPES.get (type (value))
+	
+	# Might be a subclass
+	if w_func is None:
+		for mapper_type, mapper in CONTAINER_TYPES.items ():
+			if isinstance (value, mapper_type):
+				w_func = mapper
+				
+	if w_func:
 		return w_func (value, sort_keys, indent_string, ascii_only,
 		               coerce_keys, parent_objects, indent_level)
-	return write_basic (value, ascii_only)
+		
+	if parent_objects:
+		return write_basic (value, ascii_only)
+	raise errors.WriteError ("The outermost container must be an array or object.")
 	
 def write (value, sort_keys = False, indent = None, ascii_only = True,
            coerce_keys = False):
